'use strict';

var _path = require('path');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _config = require('../templates/config');

var _config2 = _interopRequireDefault(_config);

var _yargs = require('yargs');

var _leftPad = require('left-pad');

var _leftPad2 = _interopRequireDefault(_leftPad);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _mapStream = require('map-stream');

var _mapStream2 = _interopRequireDefault(_mapStream);

var _vinylFs = require('vinyl-fs');

var _vinylFs2 = _interopRequireDefault(_vinylFs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var args = _yargs.argv['_'];

function info(type, message) {
  console.log(_chalk2.default.green.bold((0, _leftPad2.default)(type, 12)) + '  ' + message);
}

function formatTemplet(file, cb) {
  if (file._contents) {
    var _contents = file._contents.toString();
    _contents = _contents.replace(/__Name__/gi, args[2]);
    file._contents = Buffer.from(_contents, 'utf-8');
    // info('create', file.path);
  }
  cb(null, file);
};

function error(message) {
  console.error(_chalk2.default.red(message));
}

function success(message) {
  console.error(_chalk2.default.green(message));
}

function exit() {
  process.exit();
};

var action = args[1];
var step = 0,
    filesLength = 0; // 步骤
var commands = [];
Object.keys(_config2.default).forEach(function (key, i) {
  commands.push(key);
});

function help() {

  info('Usage: npm run add <command> <name>', '\n  \n  <command>:\n\n    ' + commands.join(', ') + '\n\n  <name>:\n\n    \u65B0\u5EFA\u7EC4\u4EF6\u3001\u8DEF\u7531\u3001Action\u3001Reducer\u7B49\u7684\u540D\u79F0\n  \n  Example\uFF1A\n\n    npm run add ' + commands[0] + ' demo\n  ');
}

if (args[0] === 'help' || args[0] === 'h') {
  help();
  exit();
}

if (!_config2.default[action]) {
  help();
  exit();
}
if (!args[1]) {
  error('缺少指定参数');
  exit();
}

var cwd = (0, _path.join)(__dirname, _config2.default[action].target);

/**
 * 多文件处理
 * @param {*源目录} srcDir 
 * @param {*目标目录} targetDir 
 * @param {*新增文件夹名称} name 
 */
function handleFiles(src, target, name) {
  _vinylFs2.default.src([src + '/**/*']).pipe((0, _mapStream2.default)(formatTemplet)).pipe(_vinylFs2.default.dest((0, _path.join)(target, name))).on('end', function () {
    success('\nSuccess! Created ' + name + ' at ' + target + '.\n');
    exit();
  });
}
/**
 * 单文件处理
 * @param {*源文件} src 
 * @param {*目标文件} target 
 * @param {*新增文件} name 
 */
function handleFile(src, target, name) {
  // 设置编码格式
  _fs2.default.readFile(src, 'utf-8', function (err, data) {
    // 读取文件失败/错误
    if (err) {
      throw err;
    }
    // 读取文件成功
    _fs2.default.writeFile(target, data, function (err) {
      if (err) {
        throw err;
      }
      success('\nSuccess! Created ' + name + ' at ' + target + '.\n    ');
      exit();
    });
  });
}

// 处理不同步骤
function handle(input) {
  var _Config$action = _config2.default[action],
      src = _Config$action.src,
      target = _Config$action.target;


  var addName = args[2];

  input = input && input.trim() || '';
  switch (step) {
    case 0:
      {
        var files = _fs2.default.readdirSync(src).map(function (file) {
          return file;
        });
        filesLength = files.length;
        // 多文件模板和单文件模板
        if (filesLength > 1) {
          if (!_fs2.default.existsSync(target + '/' + addName)) {
            _fs2.default.mkdirSync(target + '/' + addName);
            handleFiles(src, target, addName);
          } else {
            info('目录已经存在，是否覆盖，之前内容将无法恢复？(Y/N)', '\n  ' + (0, _path.join)(cwd, addName));
            step = 1;
          }
        } else {
          if (!_fs2.default.existsSync(target + '/' + addName + '.js')) {
            handleFile((0, _path.join)(src, 'index.js'), (0, _path.join)(target, addName + '.js'), addName);
          } else {
            info('文件已经存在，是否覆盖，之前内容将无法恢复？(Y/N)', '\n  ' + (0, _path.join)(cwd, addName + '.js'));
            step = 1;
          }
        }
      }
      break;
    case 1:
      {
        if (input === 'Y' || input === 'y') {
          info('', '\n\u6A21\u677F\u6587\u4EF6\u751F\u6210\u4E2D\uFF0C\u8BF7\u7A0D\u540E...\n        ');
          if (filesLength > 1) {
            handleFiles(src, target, addName);
          } else {
            handleFile((0, _path.join)(src, 'index.js'), (0, _path.join)(target, addName + '.js'), addName);
          }
        } else {
          // 放弃操作
          exit();
        }
      }
  }
};

process.stdin.setEncoding('utf8');

process.stdin.on('readable', function () {
  var chunk = process.stdin.read();
  if (chunk !== null) {
    handle(chunk);
  }
});
handle();
process.stdin.on('end', function () {
  info('任务结束', '');
});